//  Linrad's 'host control' routines for Si570 USB devices
//  They are designed to work with: 
//   -the 'QRP2000 USB-Controlled Synthesizer' kit from SDR-kits.net
//   -the 'Lite + USB Xtall V9.0' kit or the 'RX Ensemble Receiver' kit
//    from Tony Parks (see http://www.kb9yig.com )
//   -the Mobo v4.3.4, Mobo v3.6 and the UBW 
//   
//  This code is based on usbsoftrock.c by Andrew Nilsson (andrew.nilsson@gmail.com),
//  powerSwitch.c by Christian Starkjohann,
//  and usbtemp.c by Mathias Dalheimer of Objective Development Software GmbH (2005)
//  (see http://www.obdev.at/avrusb)
//
//  Thanks to Leif Asbrink/SM5BSZ for the support and to Sid Boyce/G3VBV for the testing.
// 
//  Use at your own risk and enjoy
//  Pierre Vanhoucke / ON5GN
//
//  UPDATE HISTORY
//  22 August 2010	function si570_rx_freq_control: 
//                      -Avoid overhead: do not open and close the si570 each time a new passband-center-freq is selected
//                       but use si570_open_switch.
//                      -Check returncode of setFreqByValue. If error, ask for a retry or a restart of Linrad .

#if LIBUSB_PRESENT == 1
#include "globdef.h"
#include "uidef.h" 
#include "screendef.h"
#include <string.h>
#include <usb.h>
#include <math.h>

#define USB_SUCCESS	        0
#define USB_ERROR_NOTFOUND      1
#define USB_ERROR_ACCESS        2
#define USB_ERROR_IO            3
#define SI570_USB_LIST_EMPY     4
#define VENDOR_NAME	        "www.obdev.at"
#define PRODUCT_NAME            "DG8SAQ-I2C"
#define USBDEV_SHARED_VENDOR    0x16C0  // VOTI 
#define USBDEV_SHARED_PRODUCT   0x05DC  // Obdev's free shared PID 
                                        // Use obdev's generic shared VID/PID pair 
                                        // and follow the rules outlined in 
                                        // firmware/usbdrv/USBID-License.txt.
#define REQUEST_READ_VERSION	  0x00
#define REQUEST_SET_FREQ_BY_VALUE 0x32

#define MAX_SI570_PARM 7
char *si570_parfil_name="par_si570";
char *si570_parm_text[MAX_SI570_PARM]={"USB index",
                                       "Min receive freq",
                                       "Max receive freq",
                                       "LO multiplier",
                                       "LO freq offset",
                                       "Passband direction",
                                       "Passband increment"
                                       }; 
typedef struct {
int USB_index;
int min_receive_freq;
int max_receive_freq;
int LO_multiplier;
int LO_freq_offset;
int passband_direction;
int passband_increment;
}P_SI570;
P_SI570 si570;

char   serialNumberString[20];
int    i2cAddress = 0x55;
char   * my_usbSerialID = NULL; 
char   work_string[80];

usb_dev_handle      *global_si570usb_handle = NULL;
int         si570_open_switch =0;
static int  didUsbInit = 0;

double round(double x);

unsigned short readVersion(usb_dev_handle *handle) 
  {
  unsigned short version;
  int nBytes;
  nBytes = usb_control_msg(handle, USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN,
         REQUEST_READ_VERSION, 0x0E00, 0, (char *) &version, sizeof(version), 500);
  if (nBytes == 2) 
    {
//    printf("Firmware version     : %d.%d\n", (version & 0xFF00) >> 8, version & 0xFF);
    return version;
    } 
  else
    {
//    printf("Firmware version     : UNKNOWN\n");
    return 0;
    }
  }

int  usbGetStringAscii(usb_dev_handle *dev, int my_index, int langid, char *buf, int buflen)
{
  char    buffer[256];
  int     rval, i;
  if((rval = usb_control_msg(dev, USB_ENDPOINT_IN, USB_REQ_GET_DESCRIPTOR, 
     (USB_DT_STRING << 8) + my_index, langid, buffer, sizeof(buffer), 1000)) < 0) return rval;
  if(buffer[1] != USB_DT_STRING)  return 0;
  if((unsigned char)buffer[0] < rval) rval = (unsigned char)buffer[0];
  rval /= 2;
  // lossy conversion to ISO Latin1 
  for(i=1;i<rval;i++)
    {
    if(i > buflen) break;                       // destination buffer overflow 
    buf[i-1] = buffer[2 * i];
    if(buffer[2 * i + 1] != 0)  buf[i-1] = '?'; // outside of ISO Latin1 range 
    }
  buf[i-1] = 0;
  return i-1;
}

unsigned char Si570usbOpenDevice(usb_dev_handle **device, char *usbSerialID)
{
  struct usb_bus      *bus;
  struct usb_device   *dev;
  usb_dev_handle      *handle = NULL;
  unsigned char       errorCode = USB_ERROR_NOTFOUND;
  char                string[256];
  int                 len;
  int  vendor        = USBDEV_SHARED_VENDOR;
  char *vendorName   = VENDOR_NAME;
  int  product       = USBDEV_SHARED_PRODUCT;
  char *productName  = PRODUCT_NAME;

  if(!didUsbInit)
    {
    didUsbInit = 1;
    usb_init();
    }
  usb_find_busses();
  usb_find_devices();
  for(bus=usb_get_busses(); bus; bus=bus->next){
    for(dev=bus->devices; dev; dev=dev->next){
      if(dev->descriptor.idVendor == vendor && dev->descriptor.idProduct == product)
        {
        handle = usb_open(dev); // we need to open the device in order to query strings 
        if(!handle)
          {
          errorCode = USB_ERROR_ACCESS;
          printf("Warning: cannot open Si570-USB device:\n");
          printf("%s\n",usb_strerror());
          continue;
          }
        if(vendorName == NULL && productName == NULL)
          {  // name does not matter 
          break;
          }
        // now check whether the names match
        len = usbGetStringAscii(handle, dev->descriptor.iManufacturer, 0x0409, string, sizeof(string));
        if(len < 0)
          {
          errorCode = USB_ERROR_IO;
          printf("Warning: cannot query manufacturer for Si570-USB device:\n");
          printf("%s\n",usb_strerror());
          }
        else
          {
          errorCode = USB_ERROR_NOTFOUND;
           //fprintf(stderr, "seen device from vendor ->%s<-\n", string); 
          if(strcmp(string, vendorName) == 0){
            len = usbGetStringAscii(handle, dev->descriptor.iProduct, 0x0409, string, sizeof(string));
            if(len < 0)
              {
              errorCode = USB_ERROR_IO;
              printf("Warning: cannot query product for Si570-USB device: \n");
              printf("%s\n",usb_strerror());
              }
            else
              {
              errorCode = USB_ERROR_NOTFOUND;
               //fprintf(stderr, "seen product ->%s<-\n", string); 
              if(strcmp(string, productName) == 0) 
                {
		len = usbGetStringAscii(handle, dev->descriptor.iSerialNumber, 0x0409, 
                      serialNumberString, sizeof(serialNumberString));
		if (len < 0) 
                  {
		  errorCode = USB_ERROR_IO;
		  printf( "Warning: cannot query serial number for Si570-USB device: \n");
                  printf("%s\n",usb_strerror());
		  }
                else
                  {
		  errorCode = USB_ERROR_NOTFOUND;
		  if ((usbSerialID == NULL) || (strcmp(serialNumberString, usbSerialID) == 0))
                    {
//                    printf("\nOpen Si570 USB device: OK\n");
//                    printf("usbSerialID          : %s\n",serialNumberString);
		    break;
		    }
		   }
	         }
               }
             }
           }
        usb_close(handle);
        handle = NULL;
      }
    }
    if(handle) break;
  }
  if(handle != NULL)
    {
    errorCode = USB_SUCCESS;
    *device = handle;
    }
  return errorCode;
}

void setLongWord( int value, char * bytes)
{
bytes[0] = value & 0xff;
bytes[1] = ((value & 0xff00) >> 8) & 0xff;
bytes[2] = ((value & 0xff0000) >> 16) & 0xff;
bytes[3] = ((value & 0xff000000) >> 24) & 0xff;
} 

int setFreqByValue(usb_dev_handle * handle, double frequency)
{

// Windows Doc from PE0FKO:
// 
// Command 0x32:
// -------------
// Set the oscillator frequency by value. The frequency is formatted in MHz
// as 11.21 bits value. 
// The "automatic band pass filter selection", "smooth tune", "one side calibration" and
// the "frequency subtract multiply" are all done in this function. (if enabled in the firmware)
// 
// Default:    None
// 
// Parameters:
//     requesttype:    USB_ENDPOINT_OUT
//     request:         0x32
//     value:           0
//     index:           0
//     bytes:           pointer 32 bits integer
//     size:            4
//
// Code sample:
//     uint32_t iFreq;
//     double   dFreq;
// 
//     dFreq = 30.123456; // MHz
//     iFreq = (uint32_t)( dFreq * (1UL << 21) )
//     r = usbCtrlMsgOUT(0x32, 0, 0, (char *)&iFreq, sizeof(iFreq));
//     if (r < 0) Error
//
	
char   buffer[4];
int    request = REQUEST_SET_FREQ_BY_VALUE;
int    value = 0x700 + i2cAddress;
int    my_index = 0;
int    retval;
setLongWord(round(frequency * 2097152.0), buffer);  //   2097152=2^21 
retval=usb_control_msg(handle, USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_OUT,
        request, value, my_index, buffer, sizeof(buffer), 5000);
return retval;

}

unsigned char Si570usbListDevices(int *line)
{
  (void) line;
  struct usb_bus      *bus;
  struct usb_device   *dev;
  usb_dev_handle      *handle = NULL;
  unsigned char       errorCode = USB_ERROR_NOTFOUND;
  int                 vendor        = USBDEV_SHARED_VENDOR;
  char                *vendorName   = VENDOR_NAME;
  int                 product       = USBDEV_SHARED_PRODUCT;
  char                *productName  = PRODUCT_NAME;
  int                 k=0;
  int                 id;
  char                s[120];
  char                s1[120];
  char                string[256];
  int                 len;
  unsigned short      version;
  char                serialIDS[16][20];

  if(!didUsbInit)
    {
    didUsbInit = 1;
    usb_init();
    }
  lir_text(6,line[0],"Select a 'fixed predefined Si570 USB device' from list:");
  line[0]+=2;
  usb_find_busses();
  usb_find_devices();
  for(bus=usb_get_busses(); bus; bus=bus->next)
    {
    for(dev=bus->devices; dev; dev=dev->next)
      {
      if(dev->descriptor.idVendor == vendor && dev->descriptor.idProduct == product)
        {
        handle = usb_open(dev); // we need to open the device in order to query strings 
        if(!handle)
          {
          errorCode = USB_ERROR_ACCESS;
          printf("Warning: cannot open USB device: %s\n", usb_strerror());
          continue;
          }
        if(vendorName == NULL && productName == NULL)
          {  // name does not matter 
          break;
          }
        // now check whether the names match
        len = usbGetStringAscii(handle, dev->descriptor.iManufacturer, 0x0409, string, sizeof(string));
        if(len < 0)
          {
          errorCode = USB_ERROR_IO;
          printf( "Warning: cannot query manufacturer for device: %s\n", usb_strerror());
          }
        else
          {
          errorCode = USB_ERROR_NOTFOUND;
 //          printf("seen device from vendor ->%s<-\n", string); 
          if(strcmp(string, vendorName) == 0)
            {
            len = usbGetStringAscii(handle, dev->descriptor.iProduct, 0x0409, string, sizeof(string));
            if(len < 0)
              {
              errorCode = USB_ERROR_IO;
              printf("Warning: cannot query product for device: %s\n", usb_strerror());
              }
            else
              {
              errorCode = USB_ERROR_NOTFOUND;
//               printf("seen product ->%s<-\n", string); 
              if(strcmp(string, productName) == 0) 
                {
		len = usbGetStringAscii(handle, dev->descriptor.iSerialNumber, 0x0409, 
                      serialNumberString, sizeof(serialNumberString));
		if (len < 0) 
                  {
		  errorCode = USB_ERROR_IO;
		  printf( "Warning: cannot query serial number for device: %s\n", usb_strerror());
		  }
                else
                  {
		  errorCode = USB_ERROR_NOTFOUND;
 //                 printf("seen serialNumberString :%s\n", serialNumberString);
                  version=readVersion(handle);
                  sprintf(s1,"( Firmware version %d.%d )", (version & 0xFF00) >> 8, version & 0xFF);
                  sprintf(s,"%2d ->  serialID = %s %s", k, serialNumberString, s1);
                  sprintf(serialIDS[k],"%s", serialNumberString);
                  lir_text(6,line[0],s);
                  line[0]++;
                  k++; 
                  if (k == 16) goto exit;  // array for serialIDS is full   
		  }
	         }
            }
          }
        }
        usb_close(handle);
        handle = NULL;
      }
    }
    if(handle) break;
  }
exit:
  if(handle != NULL)
    {
    errorCode = USB_SUCCESS;
    usb_close(handle);
    }
  if (k ==0 )
    {
    line[0]=line[0]-2;
    settextcolor(12);
    lir_text(6,line[0],
    "No Si570 USB devices found                                        ");
    settextcolor(7);
    line[0]++;
    return SI570_USB_LIST_EMPY;
    }
  line[0]++;
  sprintf(s,"Enter line number >");
  lir_text(6,line[0],s);
  id=lir_get_integer(26,line[0],2, 0, k-1);
  if(kill_all_flag) return errorCode;
  sprintf(work_string,"%s",serialIDS[id]);
  my_usbSerialID=work_string;
  return errorCode;
}

//  End Linrad's 'host control' routines for Si570 USB devices

void set_default_parameters(void)
{
// SET DEFAULTS VALUES FOR RX Ensemble Receiver kit
si570.USB_index        =0;  // DYNAMIC SELECTION OF SI570 USB DEVICE
si570.LO_multiplier    =4;
// set frequency defaults 
// **********************************************************************
//  From www.wb5rvz.com:
//  The RX Ensemble Receiver kit provides coverage of ham bands from 160-10m, 
//  in four different optional "super bands" (each with underlap 
//  and overlap within the parameters of the associated bandpass filter):
//  1. 160m - Continuous coverage from 1.8 to 2.0 MHz
//  2. 80m and 40m - Continuous coverage from 3.5 to 7.3 MHz
//  3. 30m, 20m, and 17m - Continuous coverage from 10.1 to 18.168 MHz
//  4. 15m, 12m, and 10m - Continuous coverage from 21.0 to 29.7 MHz
//
// **********************************************************************
si570.min_receive_freq  =1800;
si570.max_receive_freq  =29700;
si570.LO_freq_offset    =0;
si570.passband_direction=1;
si570.passband_increment=45;
}

void read_par_si570_file(void)
{
int errcod;
errcod=read_sdrpar(si570_parfil_name, MAX_SI570_PARM, 
                                     si570_parm_text, (int*)((void*)&si570));
if (errcod != 0)
{
FILE *si570_file;
int *sdr_pi,i;
si570_file=fopen(si570_parfil_name,"w");
set_default_parameters();
sdr_pi=(void*)(&si570);
for(i=0; i<MAX_SI570_PARM; i++)
  {
  fprintf(si570_file,"%s [%d]\n",si570_parm_text[i],sdr_pi[i]);
  }
parfile_end(si570_file);
}
if ( si570.USB_index==0 ) my_usbSerialID=NULL;
if ( si570.USB_index==1 )
  {
  int ir;
  FILE *Fp;
  char si570_usb_id_file[20];
  sprintf(si570_usb_id_file,"par_si570_usb_id");
  Fp=fopen(si570_usb_id_file,"r");
  ir=fscanf(Fp,"%20s\n",work_string);
  my_usbSerialID=work_string;
  fclose(Fp);
  }
}

void save_par_si570_file(void)
{
FILE *si570_file;
int *sdr_pi,i;
si570_file=fopen(si570_parfil_name,"w");
sdr_pi=(void*)(&si570);
for(i=0; i<MAX_SI570_PARM; i++)
  {
  fprintf(si570_file,"%s [%d]\n",si570_parm_text[i],sdr_pi[i]);
  }
parfile_end(si570_file);
}

void save_par_si570_usb_id_file(void)
{
FILE *Fp;
char si570_usb_id_file[20];
sprintf(si570_usb_id_file,"par_si570_usb_id");
Fp=fopen(si570_usb_id_file,"w");
fprintf(Fp,"%20s\n", my_usbSerialID );
fclose(Fp);
}

void si570_setup(void)
{
char s[160];
char s1[80];
int  retval;
int  line;
usb_dev_handle	*handle = NULL;
if (si570_open_switch == 1)  usb_close(global_si570usb_handle);
si570_open_switch = 0  ;                               //force open next time si570_rx_freq_control gets called 
start_setup:
clear_screen();
settextcolor(14);
lir_text(2,1,"CURRENT SETUP FOR 'SI570 USB DEVICE' ");
settextcolor(7);
line= 4;
read_par_si570_file();
if (my_usbSerialID != NULL)   
// STATIC
  {
  sprintf(s,"USB serialID                      =");
  lir_text(2,line,s);
  retval=Si570usbOpenDevice(&handle, my_usbSerialID);
  if(retval == 0)
    {
    retval=readVersion(handle);
    sprintf(s1,"%d.%d", (retval & 0xFF00) >> 8, retval & 0xFF);
    usb_close(handle);
    settextcolor(10);
    sprintf(s,"%s (Firmware version %s)",my_usbSerialID,s1);
    lir_text(38,line,s);
    settextcolor(7);
    line+=2;
    }
  else
    {
    settextcolor(12);
    sprintf(s,"%s",my_usbSerialID);
    lir_text(38,line,s);
    sprintf(s,"could not open device, device probably not connected");
    lir_text(58,line,s);
    settextcolor(7);
    line+=2;
    }
  sprintf(s,"USB device selection mode         = ");
  lir_text(2,line,s);
  settextcolor(10);
  sprintf(s,"STATIC ( => Linrad uses a fixed predefined device )");
  lir_text(38,line,s);
  settextcolor(7);
  line+=2;
  }
else
//DYNAMIC  (my_usbSerialID == NULL) 
  {
  sprintf(s,"USB serialID                      =");
  lir_text(2,line,s);
  retval=Si570usbOpenDevice(&handle, my_usbSerialID);
  if(retval != 0)
   {
   sprintf(s,"Could not detect a Si570 USB device, probably NO device connected");
   settextcolor(12);
   lir_text(38,line,s);
   settextcolor(7);
   line+=2;
   }
  else
   {
   retval=readVersion(handle);
   usb_close(handle);
   sprintf(s1,"%d.%d", (retval & 0xFF00) >> 8, retval & 0xFF);
   sprintf(s,"%s (Firmware version %s)",serialNumberString, s1);
   settextcolor(10);
   lir_text(38,line,s);
   settextcolor(7);
   line+=2;
   }
  sprintf(s,"USB device selection mode         = ");
  lir_text(2,line,s);
  settextcolor(10);
  sprintf(s,"DYNAMIC ( => Linrad uses the first detected device )");
  lir_text(38,line,s);
  settextcolor(7);
  line+=2;
  }
line+=2;
sprintf(s,"Si570 min receive frequency (Khz) = ");
lir_text(2,line,s);
settextcolor(10);
sprintf(s,"%i",si570.min_receive_freq);
lir_text(38,line,s);
settextcolor(7);
line+=2;
sprintf(s,"Si570 max receive frequency (Khz) = ");
lir_text(2,line,s);
settextcolor(10);
sprintf(s,"%i",si570.max_receive_freq);
lir_text(38,line,s);
settextcolor(7);
line+=2;
sprintf(s,"Si570 LO frequency multiplier     = ");
lir_text(2,line,s);
settextcolor(10);
sprintf(s,"%i",si570.LO_multiplier);
lir_text(38,line,s);
settextcolor(7);
line+=2;
sprintf(s,"Si570 LO frequency offset (Khz)   = ");
lir_text(2,line,s);
settextcolor(10);
sprintf(s,"%i",si570.LO_freq_offset);
lir_text(38,line,s);
settextcolor(7);
line+=2;
sprintf(s,"Si570 resulting LO range (Khz)    = ");
lir_text(2,line,s);
settextcolor(10);
sprintf(s,"from %i to %i",
(si570.min_receive_freq*si570.LO_multiplier)-si570.LO_freq_offset,
(si570.max_receive_freq*si570.LO_multiplier)-si570.LO_freq_offset
);
lir_text(38,line,s);
settextcolor(7);
// error checking
int si570_freq_error=0;
if ((si570.min_receive_freq*si570.LO_multiplier)-si570.LO_freq_offset < 3500 )
  {
  si570_freq_error=1;
  line+=2;
  settextcolor(12);
  lir_text(38,line,"Invalid 'from' frequency: should be >= 3500 Khz ! ");
  settextcolor(7);
  }
if ((si570.min_receive_freq*si570.LO_multiplier)-si570.LO_freq_offset > 945000 )
 {
 si570_freq_error=1;
 line+=2;
 settextcolor(12);
 lir_text(38,line,"Invalid 'from' frequency:   should be <= 945000 Khz ! ");
 settextcolor(7);
 }
if ((si570.max_receive_freq*si570.LO_multiplier)-si570.LO_freq_offset < 3500 )
 {
 si570_freq_error=1;
 line+=2;
 settextcolor(12);
 lir_text(38,line,"Invalid 'to' frequency: should be >= 3500 Khz ! ");
 settextcolor(7);
 }
if ((si570.max_receive_freq*si570.LO_multiplier)-si570.LO_freq_offset > 945000 )
 {
 si570_freq_error=1;
 line+=2;
 settextcolor(12);
 lir_text(38,line,"Invalid 'to' frequency: should be <= 945000 Khz ! ");
 settextcolor(7);
 }
if (si570_freq_error==1)
  {
  line+=2;
  settextcolor(12);
  lir_text(38,line,"CHECK FREQUENCY PARAMETERS");
  settextcolor(7);
  }
// warning
if (si570_freq_error==0)
  { 
  if ((si570.max_receive_freq*si570.LO_multiplier)-si570.LO_freq_offset > 210000 )
   {
   line+=2;
   settextcolor(13);
   lir_text(38,line,"Make sure your Si570 chip can handle frequencies > 210000 Khz !");
   settextcolor(7);
   }
  }
line+=2;
line+=2;
sprintf(s,"Si570 passband direction          = ");
lir_text(2,line,s);
settextcolor(10);
sprintf(s,"%i", si570.passband_direction );   
lir_text(38,line,s);
settextcolor(7);
line+=2;
sprintf(s,"Si570 passband increment (Khz)    = ");
lir_text(2,line,s);
settextcolor(10);
sprintf(s,"%i",si570.passband_increment );
lir_text(38,line,s);
settextcolor(7);
line+=6;
lir_text(2,line,"A = Set selection-mode to DYNAMIC.");
line++;
lir_text(2,line,"B = Set selection-mode to STATIC and select a 'predefined Si570 USB device'.");
line++;
line++;
lir_text(2,line,"C = Change MIN RECEIVE frequency.");
line++;
lir_text(2,line,"D = Change MAX RECEIVE frequency.");
line++;
lir_text(2,line,"E = Change LO FREQUENCY MULTIPLIER.");
line++;
lir_text(2,line,"F = Change LO FREQUENCY OFFSET.");
line++;
line++;
lir_text(2,line,"H = Change PASSBAND DIRECTION.");
line++;
lir_text(2,line,"I = Change PASSBAND INCREMENT.");
line++;
line++;
lir_text(2,line,"J = RESTORE default parameters (for RX Ensemble Receiver kit).");
line++;
lir_text(2,line,"X = Exit.");

setup_menu:;
await_processed_keyboard();
if(kill_all_flag)return;
switch(lir_inkey)
  {

  case 'A':
    my_usbSerialID = NULL;
    si570.USB_index=0;
    save_par_si570_file();
    goto start_setup;
  break;

  case 'B':
     line= line+4;
     settextcolor(11);
     lir_text(6,line,
     "SETTING USB DEVICE SELECTION MODE TO STATIC:");
     line+=1;
     lir_text(6,line,
     "Looking for available Si570 USB devices:");
     line+=2;
     settextcolor(7);
     retval=Si570usbListDevices(&line);
     if (retval == SI570_USB_LIST_EMPY )
       {
       line= line+2;
       lir_text(6,line,"Hit any key to go back ");
       await_processed_keyboard();
       goto start_setup;
       } 
     si570.USB_index=1;
     save_par_si570_file();
     save_par_si570_usb_id_file();
     goto start_setup;
  break;

  case 'C':
    line=line+4;
    settextcolor(11);
    lir_text(6,line,
    "CHANGING MIN RECEIVE FREQUENCY:");
    line+=1;
    lir_text(6,line,
    "Make sure that ((min receive frequency * LO freq multiplier) - LO freq offset) >= 3500 Khz ");
    line+=1;
    lir_text(6,line,
    "3500 Khz being the lower frequency limit for Si570 devices.");
    line+=2;
    settextcolor(7);
    lir_text(6,line,"Enter new value for min receive frequency in Khz >");
    si570.min_receive_freq=lir_get_integer(57, line, 6, 0, 945000);
    if(kill_all_flag) return;
    if (si570.min_receive_freq >= si570.max_receive_freq ) si570.min_receive_freq = si570.max_receive_freq ;
    save_par_si570_file();
    goto start_setup;
  break;

  case 'D':
    line=line+4;
    settextcolor(11);
    lir_text(6,line,
    "CHANGING MAX RECEIVE FREQUENCY:");
    line+=1;
    lir_text(6,line,
    "Make sure that ((max receive frequency * LO freq multiplier) - LO freq offset) ");
    line+=1;
    lir_text(6,line,
    "is less than the upper frequency limit of your Si570 device.");
    line+=2;
    settextcolor(7);
    lir_text(6,line,"Enter new value for max receive frequency in Khz >");
    si570.max_receive_freq=lir_get_integer(57, line, 6, 0, 945000);
    if(kill_all_flag) return;
    if (si570.max_receive_freq <= si570.min_receive_freq ) si570.max_receive_freq = si570.min_receive_freq ;
    save_par_si570_file();
    goto start_setup;
  break;

  case 'E':
    line=line+4;
    settextcolor(11);
    lir_text(6,line,
    "CHANGING LO MULTIPLIER:");
    line+=1;
    lir_text(6,line,
    "Use 4 for receivers with QSD mixers like Softrock or 1 for receivers with normal mixers");
    line+=2;
    settextcolor(7);
    lir_text(6,line,"Enter new LO multiplier (1 or 4) >");
again_multiplier:;
    si570.LO_multiplier=lir_get_integer(41, line, 1, 1, 4);
    if(kill_all_flag) return;
    if ((si570.LO_multiplier==2) | (si570.LO_multiplier==3)) goto again_multiplier;
    save_par_si570_file();
    goto start_setup;
  break;

  case 'F':
    line=line+4;
    settextcolor(11);
    lir_text(6,line,
    "CHANGING LO FREQUENCY OFFSET:");
    line+=1;
    lir_text(6,line,
    "Use 0 for receivers with zero IF like Softrock, or plus or minus the IF-value for non-zero-IF receivers ");
    line+=2;
    settextcolor(7);
    lir_text(6,line,"Enter new value for LO frequency offset in Khz >");
    si570.LO_freq_offset=lir_get_integer(55, line, 7, -945000, 945000);
    if(kill_all_flag) return;
    save_par_si570_file();
    goto start_setup;
  break;

  case 'H':
    line=line+4;
    settextcolor(11);
    lir_text(6,line,
    "CHANGING PASSBAND DIRECTION:");
    line+=1;
    lir_text(6,line,
    "Toggling between 1 and -1  allows to reverse the spectrum displayed on the waterfall graph");
    line+=2;
    settextcolor(7);
again_passband_direction:;
    lir_text(6,line,"Enter new value for passband direction ( 1 or -1 ) >");
    si570.passband_direction=lir_get_integer(60, line, 2, -1, 1);
    if(kill_all_flag) return;
    if (si570.passband_direction ==0) goto again_passband_direction;
    save_par_si570_file();
    goto start_setup;
  break;

  case 'I':
    line=line+4;
    settextcolor(11);
    lir_text(6,line,
    "CHANGING PASSBAND INCREMENT:");
    line+=1;
    lir_text(6,line,
    "Set this value near the sampling rate of your soundcard when in I/Q mode");
    line+=1;
    lir_text(6,line,
    "For normal audio mode use a value near (sampling rate)/2");
    line+=2;
    settextcolor(7);
    lir_text(6,line,"Enter new value for passband increment in Khz >");
    si570.passband_increment=lir_get_integer(57, line, 10, 1, 192);
    if(kill_all_flag) return;
    save_par_si570_file();
    goto start_setup;
  break;

  case 'J':
    set_default_parameters();
    save_par_si570_file();
    goto start_setup;
  break;

  case 'X':
  return;

  default:
  goto setup_menu;
  }
}

void si570_rx_freq_control(void)
{
int  retval;
double dt1;
char s[120];
start_retry:;
read_par_si570_file();
fg.passband_direction=si570.passband_direction; 
fg.passband_increment=si570.passband_increment*.001;
if (fg.passband_center < si570.min_receive_freq*.001 )
  {
  fg.passband_center = si570.min_receive_freq*.001 ;
  }
if (fg.passband_center > si570.max_receive_freq*.001 )
  {
  fg.passband_center = si570.max_receive_freq*.001 ;
  }
if (si570_open_switch == 1) goto set_freq; 
retval=Si570usbOpenDevice(&global_si570usb_handle, my_usbSerialID);
dt1=current_time();
while(retval !=0)
  {
  if(retval == USB_ERROR_NOTFOUND )
    {
    clear_lines ( screen_last_line, screen_last_line );
    settextcolor(12);
    if (my_usbSerialID == NULL)
    sprintf(s,"No Si570 USB device found, please connect a device: Waiting %.2f                 ",
                                                           current_time()-dt1);
    else
    sprintf(s,
    "Could not open Si570 USB device with serialID = %s, please connect this device: Waiting %.2f",
                                            my_usbSerialID, current_time()-dt1);
    lir_text(0,screen_last_line,s);
    settextcolor(7);
    lir_refresh_screen();
    lir_sleep(50000);
    retval=Si570usbOpenDevice(&global_si570usb_handle, my_usbSerialID);
    if(kill_all_flag)return; 
    }
  else
   {
    clear_lines ( screen_last_line, screen_last_line );
    if ( retval == USB_ERROR_ACCESS )
    sprintf(s,
      "ERROR_WHEN SEARCHING Si570-USB DEVICE: HIT ENTER TWICE TO RETRY OR ESCAPE KEY TO EXIT");
    if ( retval == USB_ERROR_IO )
    sprintf(s,
      "IO ERROR WHEN ACCESSING Si570-USB DEVICE: HIT ENTER TWICE TO RETRY OR ESCAPE KEY TO EXIT");
    settextcolor(12);
    lir_text(15,screen_last_line,s);
    settextcolor(7);
    lir_refresh_screen();
    await_processed_keyboard();
    if(kill_all_flag)return;
    else retval=Si570usbOpenDevice(&global_si570usb_handle, my_usbSerialID);
   }                         
  }
si570_open_switch = 1;
clear_lines ( screen_last_line, screen_last_line );
lir_refresh_screen();
set_freq:; 
retval=setFreqByValue(global_si570usb_handle,(fg.passband_center*si570.LO_multiplier)-(si570.LO_freq_offset*.001));
if (retval != 4 ) 
  {
  clear_lines ( screen_last_line, screen_last_line );
  settextcolor(12);
  sprintf(s,
        "USB ERROR WHEN SETTING NEW PASSBAND-CENTER-FREQUENCY: HIT ENTER TWICE TO RETRY OR ESCAPE KEY TO EXIT");
  lir_text(15,screen_last_line,s);
  settextcolor(7);
  lir_refresh_screen();
  await_processed_keyboard();
  if(kill_all_flag)return;
  else
    {
    usb_close( global_si570usb_handle);
    si570_open_switch = 0;
    goto start_retry;
    }
  }
}
#endif
